﻿using MathNet.Numerics.Statistics;
using Microsoft.ML;
using Microsoft.ML.Data;
using Microsoft.ML.Transforms;
using System;
using System.Collections.Generic;
using System.Linq;

class Program
{
    static void Main(string[] args)
    {
        MLContext ctx = new MLContext();

        IDataView data = ctx.Data.LoadFromTextFile<ClickData>("click.csv", separatorChar: ',', hasHeader: true);

        var xColumn = data.Schema[0];
        var yColumn = data.Schema[1];
        
        {
            Func<NormalizingEstimator> func = () => ctx.Transforms.NormalizeMeanVariance(xColumn.Name);
            ShowResult("NormalizeMeanVariance", ctx, data, func);
        }

        {
            Func<NormalizingEstimator> func = () => ctx.Transforms.NormalizeLogMeanVariance(xColumn.Name);
            ShowResult("NormalizeLogMeanVariance", ctx, data, func);
        }

        {
            Func<NormalizingEstimator> func = () => ctx.Transforms.NormalizeMinMax(xColumn.Name);
            ShowResult("NormalizeMinMax", ctx, data, func);
        }

        {
            var xData = data.GetColumn<double>(xColumn).NormalizeZscore();
            Console.WriteLine($"[NormalizeZscore] Min: {xData.Min()}, Max: {xData.Max()}");
        }
    }

    private static void ShowResult(string title, MLContext ctx, IDataView data, Func<NormalizingEstimator> func)
    {
        var transformer = func();
        ITransformer textTransformer = transformer.Fit(data);
        IDataView normalizedData = textTransformer.Transform(data);

        var xyList = ctx.Data.CreateEnumerable<ClickData>(normalizedData, false);
        var xData = xyList.Select(xy => xy.X);
        Console.WriteLine($"[{title}] Min: {xData.Min()}, Max: {xData.Max()}");
    }
}

public static class Extension
{
    public static IEnumerable<double> NormalizeZscore(this IEnumerable<double> data)
    {
        double mean = Statistics.Mean(data);
        double std = Statistics.PopulationStandardDeviation(data);

        foreach (var item in data)
        {
            yield return (item - mean) / std;
        }
    }
}

